#!/usr/bin/perl
#
# See LICENSE for copyright information
#
# check_filemod.pl <host> <filename> [warn] [critical] [port]
#
# NetSaint host script to check the number of seconds since the last
# file update from a client that is running netsaint_statd.
#

require 5.003;
BEGIN { $ENV{PATH} = '/bin' }
use Socket;
use POSIX;
use strict;

sub usage;

my $TIMEOUT = 15;

my %ERRORS = ('UNKNOWN', '-1',
		'OK', '0',
		'WARNING', '1',
		'CRITICAL', '2');
my $remote = shift || &usage(%ERRORS);
my $filename = shift || &usage(%ERRORS);
my $warn = shift || 300;		# default <warn> is 5 minutes
my $crit = shift || ($warn * 2);	# default <critical> is 10 minutes
my $port = shift || 1040;

my $remoteaddr = inet_aton("$remote");
my $paddr = sockaddr_in($port, $remoteaddr) || die "Can't create info for connection: #!\n";;
my $proto = getprotobyname('tcp');
socket(Server, PF_INET, SOCK_STREAM, $proto) || die "Can't create socket: $!";
setsockopt(Server, SOL_SOCKET, SO_REUSEADDR, 1);

# Just in case of problems, let's not hang NetSaint
$SIG{'ALRM'} = sub {
     close(Server);
     select(STDOUT);
     print "No Answer from Client\n";
     exit $ERRORS{"UNKNOWN"};
};
alarm($TIMEOUT);

# send ourselves a SIGALRM if we cannot connect to the client
unless (connect(Server, $paddr)) {
     kill 'ALRM', $$;
}

my $state = "OK";
my $answer = undef;

select(Server);
$| = 1;
print Server "filemodinterval $filename\n";
my ($servanswer) = <Server>;
alarm(0);
close(Server);
select(STDOUT);

if ($servanswer =~ /^(\d+)/)
	{
	my $modinterval = $1;
	if ($modinterval < $warn)
		{ 
		$answer = "last file update $modinterval seconds ago\n";
		}
	elsif ($modinterval < $crit)
		{
		$state = "WARNING";
		$answer = "last file update $modinterval seconds ago\n";
		}
	else
		{
		$state = "CRITICAL";
		$answer = "last file update $modinterval seconds ago\n";
		}
	}
elsif ($servanswer =~ /^pluginerror: (.*)/)
	{
	$state = "UNKNOWN";
	$answer = "$1\n";
	}
else
	{
	$state = "UNKNOWN";
	$answer = "Information unknown!\n";
	}

print $answer;
exit $ERRORS{$state};

sub usage {
	print "Minimum arguments not supplied!\n";
	print "\n";
	print "Perl Check File Modification Interval plugin for NetSaint\n";
	print "Copyright (c) 1999 Charlie Cook\n";
	print "\n";
	print "Usage: $0 <host> <filename> [<warn [<crit> [<port>]]]\n";
	print "\n";
	print "<warn> = seconds since last modification at which a warning message will be generated.\n	Defaults to 300.\n";
	print "<crit> = seconds since last modification at which a critical message will be generated.\n	Defaults to warning level * 2.\n";
	print "<port> = Port that the status daemon is running on <host>.\n	Defaults to 1040.\n";
	exit $ERRORS{"UNKNOWN"};
}
