#!/usr/bin/perl
#
# this software is released under the GPL (http://www.gnu.org/)
# it is written by Eric Lewandowski <fear-the-penguin@home.com>
# and <tmancill@debian.org>
#
# check_swap.pl <host> [warn] [critical] [port]
#
# NetSaint host script to get the %-utilization of swap from netsaint_statd.
#

$SWAPWARN = 85;
$SWAPCRITICAL = 95;

require 5.003;
BEGIN { $ENV{PATH} = '/bin' }
use Socket;
use POSIX;

sub usage;

my $TIMEOUT = 15;

my %ERRORS = ('UNKNOWN', '-1',
		'OK', '0',
		'WARNING', '1',
		'CRITICAL', '2');
my $remote = shift || &usage(%ERRORS);
my $warn = shift || $SWAPWARN;
my $crit = shift || $SWAPCRITICAL;
my $port = shift || 1040;

my $remoteaddr = inet_aton("$remote");
my $paddr = sockaddr_in($port, $remoteaddr) || die "Can't create info for connection: #!\n";;
my $proto = getprotobyname('tcp');
socket(Server, PF_INET, SOCK_STREAM, $proto) || die "Can't create socket: $!";
setsockopt(Server, SOL_SOCKET, SO_REUSEADDR, 1);

# Just in case of problems, let's not hang NetSaint
$SIG{'ALRM'} = sub {
     close(Server);
     select(STDOUT);
     print "No Answer from Client\n";
     exit $ERRORS{"UNKNOWN"};
};
alarm($TIMEOUT);

# send ourselves a SIGALRM if we cannot connect to the client
unless (connect(Server, $paddr)) {
     kill 'ALRM', $$;
}

my $state = "OK";
my $answer = undef;


select(Server);
$| = 1;
print Server "swap\n";
my ($servanswer) = <Server>;
alarm(0);
close(Server);
select(STDOUT);

if ($servanswer =~ /^(\d+)/) {
	$swaputil = $1;
	if ($swaputil < $warn) { 
		$answer = "swap ok - $swaputil\% utilized\n";
	} elsif ($swaputil < $crit) {
		$state = "WARNING";
		$answer = "swap $swaputil\% utilized\n";
	} else {
		$state = "CRITICAL";
		$answer = "swap $swaputil\% utilized\n";		
	}
			
} else {
	$state = "UNKNOWN";
	$answer = "swap utilization unknown!\n";
}

print $answer;
exit $ERRORS{$state};

sub usage {
	print "Minimum arguments not supplied!\n";
	print "\n";
	print "Perl Check Swap plugin for NetSaint\n";
	print "Copyright (c) 2001 tony mancill\n";
	print "\n";
	print "Usage: $0 <host> [<warn [<crit> [<port>]]]\n";
	print "\n";
	print "<warn> = swap utilization at which a warning message will be generated.\n	Defaults to $SWAPWARN.\n";
	print "<crit> = swap utilization at which a critical message will be generated.\n	Defaults to $SWAPCRITICAL.\n";
	print "<port> = Port that the status daemon is running on <host>.\n	Defaults to 1040.\n";
	exit $ERRORS{"UNKNOWN"};
}
